
' Algorithms to compute easter sunday month and day for any given year

VERSION "0.0001"
CONSOLE

'	IMPORT "xst"				' Standard library : required by most programs
'	IMPORT "xsx"				' Extended standard library
'	IMPORT "xio"				' Console input/ouput library

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

DECLARE FUNCTION Entry ()
DECLARE FUNCTION EaSun (Year, @Month, @Day)
DECLARE FUNCTION Easter (Year, @Month, @Day)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	Year = XLONG (INLINE$ ("Enter Year > "))
		
	EaSun (Year, @Month, @Day)
		
	PRINT "Easter Sunday for year :"; Year 
	PRINT "Month number           :"; Month 
	PRINT "Day number             :"; Day 
		
	IF Month = 3 THEN    ' 3rd month of year 
		N$ = "March" 
	ELSE                 ' must be 4th month 
		N$ = "April" 
	END IF 
	?	
	PRINT "So easter date is: "; N$; " "; STRING$(Day); ","; Year 

' compare routines for easter sunday	
'	FOR Year = 1583 TO 4201
'    Easter(Year, @Month, @Day)
'		EaSun (Year, @M, @D)
'		? Year, Month, M, Day, D
'    IF Month <> M THEN ? "Error: Month", Year, Month, M : EXIT FOR
'		IF Day <> D THEN ? "Error: Day", Year, Day, D : EXIT FOR
'  NEXT Year

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

' EaSun  - Calculates date of Easter Sunday 
  
' EaSun algorithm published in Scientific American magazine, March 
' 2001, Page 82. Earlier published by Thomas H. O'Bierne in 1965. 
' Algorithm includes all rules for calculating the date of Easter. 

FUNCTION EaSun (Year, @Month, @Day) 
  ' Returns date of Easter Sunday, in Gregorian Year "Year", 
  ' as "Month" and "Day", both as numbers. 
  A = Year MOD 19 
  B = Year \ 100 
  C = Year MOD 100 
  D = B \ 4 
  E = B MOD 4 
  G = (8 * B + 13) \ 25 
  H = (19 * A + B - D - G + 15) MOD 30 
  M = (A + 11 * H) \ 319 
  J = C \ 4 
  K = C MOD 4 
  L = (2 * E + 2 * J - K - H + M + 32) MOD 7 
  Q = H - M + L 
  Month = (Q + 90) \ 25           ' month. 3 = March. 4 = April. 
  Day = (Q + Month + 19) MOD 32   ' day of month 

END FUNCTION

'  Modified Oudin Easter Algorithm by Claus Tonderings

FUNCTION Easter (Year, @Month, @Day)

' C Century
' G Golden Number - 1
' H 23-Epact (modulo 30)
' I Days from 21 Mar to the Paschal full moon
' J Weekday for Paschal full moon (Sun=0,Mon=1,...)
' L Days from 21 Mar to Sunday <= Paschal full moon (-6 to 28)

  C = Year \ 100
  G = Year MOD 19
  H = (C - (C \ 4) - ((8 * C + 13) \ 25) + (19 * G) + 15) MOD 30
  I = H - (H \ 28) * (1 - (29 \ (H + 1)) * ((21 - G) \ 11))
  J = (Year + (Year \ 4) + I + 2 - C + (C \ 4)) MOD 7
  L = I - J
  Month = 3 + ((L + 40) \ 44)
  Day = L + 28 - (31 * (Month \ 4))

END FUNCTION
END PROGRAM