'
' ####################
' #####  PROLOG  #####
' ####################
'
' A command line driven ftp file transfer program
' based on qftp.c by Thomas Pfau
' using ftp library xinftp
'
VERSION "0.0001"
CONSOLE
'
	IMPORT "xst_s"
	IMPORT "xsx_s"
	IMPORT "xio_s"
	IMPORT "xin_s"
	IMPORT "xinftp_s"

'	IMPORT "xst"
'	IMPORT "xsx"
'	IMPORT "xio"
'	IMPORT "xin"
'	IMPORT "xinftp"
	IMPORT "msvcrt"
	IMPORT "wsock32"

' exit values 
$$EX_SYNTAX = 2 			' command syntax errors
$$EX_NETDB = 3				' network database errors
$$EX_CONNECT = 4			' network connect errors
$$EX_LOGIN = 5				' remote login errors
$$EX_REMCMD = 6				' remote command errors
$$EX_SYSERR = 7				' system call errors

$$FTP_SEND = 1				' send files
$$FTP_GET = 2					' retreive files
$$FTP_DIR = 3					' verbose directory
$$FTP_RM = 4					' delete files
$$FTP_LIST = 5				' terse directory

$$DIRBUF_SIZE = 1024 	' for wildcard processing

DECLARE FUNCTION Entry ()
DECLARE FUNCTION Usage ()
DECLARE FUNCTION FtpConnection ()
DECLARE FUNCTION GetNextArg (index, argc, @argv$[], @arg$)
DECLARE FUNCTION ProcessFile (file$)
DECLARE FUNCTION SetUmask (mask$)
DECLARE FUNCTION ChangeDir (root$)
DECLARE FUNCTION LogProgress (GIANT filesize, GIANT bytes)
DECLARE FUNCTION ProgressBar (row, column, length, min, max, value, emptyChar, filledChar)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	SHARED logged_in
	SHARED host$
	SHARED user$
	SHARED pass$
	SHARED mode
	SHARED action
	SHARED socket
	SHARED wildcard
	
	Xin ()  		' initialize Xin library
	XinFtp () 	' init XinFtp library
	
	mode = 'I'

	XstGetCommandLineArguments (@argc, @argv$[])
	action = 0
	
  IF (argc > 2) THEN

		SELECT CASE LCASE$(argv$[1])
			CASE "send" : action = $$FTP_SEND
			CASE "get"  : action = $$FTP_GET
			CASE "dir"  : action = $$FTP_DIR
			CASE "list" : action = $$FTP_LIST
			CASE "rm"   : action = $$FTP_RM
		END SELECT
		
  END IF
		
  IFZ action THEN
		Usage()
		QUIT ($$EX_SYNTAX)
	END IF
	
	' process remaining command line args
	
	FOR optindex = 2 TO UBOUND(argv$[])
	
		IF argv$[optindex]{0} != '-' THEN
			IFZ host$ THEN
				host$ = argv$[optindex]
			ELSE
				file$ = argv$[optindex]
				INC files_processed
				ProcessFile (file$)
			END IF
			DO NEXT
		END IF
	
		opt = argv$[optindex]{1}
		
		SELECT CASE opt
		
			CASE '?' : Usage() : QUIT($$EX_SYNTAX)
			CASE 'a' : mode = 'A'
			CASE 'i' : mode = 'I'
			CASE 'l' : GetNextArg (optindex, argc, @argv$[], @user$) : INC optindex
			CASE 'm' : GetNextArg (optindex, argc, @argv$[], @mask$)
								 INC optindex
								 SetUmask (mask$)
			CASE 'p' : GetNextArg (optindex, argc, @argv$[], @pass$) : INC optindex
			CASE 'r' : GetNextArg (optindex, argc, @argv$[], @root$)
								 INC optindex
								 ChangeDir (dir$)
			CASE 'e' : FtpEcho ($$TRUE)
			CASE 'w' : wildcard = $$TRUE
			CASE ELSE :	Usage() : QUIT ($$EX_SYNTAX)
		
		END SELECT
			
	NEXT 
	
	IFZ files_processed THEN
		FtpConnection ()
		IF (action == $$FTP_DIR) || (action == $$FTP_LIST) THEN
	    ProcessFile ("")
		ELSE
			IF (action == $$FTP_GET) && (wildcard = $$TRUE) THEN
				ProcessFile ("")
			ELSE
				DO
					file$ = TRIM$(INLINE$ ("file> "))
					IF file$ THEN ProcessFile (file$)
				LOOP WHILE file$
			END IF
		END IF
	END IF	
	
	IF socket THEN FtpQuit (socket)
	
END FUNCTION

FUNCTION Usage ()

	? "usage:  xftp <send | get | dir | list | rm> <hostname>"
	? "\t[ -l user [ -p pass ] ]  user login name and password"
	? "\t["
	? "\t  [ -e ]              toggle echo mode (display commands and responses)"
	? "\t  [ -r rootpath ]     chdir path"
	? "\t  [ -m umask ]        umask for created files"
	? "\t  [ -a | -i ] ]       ascii/image transfer file"
	? "\t  [ -w ]              set wildcard mode"
	? "\t  [ file ]            file spec for directory or file to transfer"
	? "\t]..."
	?
	? "If no files are specified on command line, the program"
	? "will read file names from stdin."

END FUNCTION

FUNCTION FtpConnection ()

	SHARED logged_in
	SHARED host$
	SHARED user$
	SHARED pass$
	SHARED mode
	SHARED action
	SHARED socket
	SHARED wildcard

  IF socket THEN RETURN

  IFZ host$ THEN 
		? "Host name not specified"
		Usage()
		QUIT ($$EX_SYNTAX)
	END IF
		
  IF !logged_in THEN
		IFZ user$ THEN
			? "User name not specified"
			Usage ()
			QUIT ($$EX_SYNTAX)
		END IF
		
		IFZ pass$ THEN
			? "Password not specified"
			Usage ()
			QUIT ($$EX_SYNTAX)
		END IF

    IF FtpConnect (host$, @socket) THEN
    	FtpLastResponse (@response$)
	    ? "Unable to connect to "; host$
			? response$
	    QUIT ($$EX_CONNECT)
		END IF
		
    IF FtpLogin (user$, pass$, socket) THEN
    	FtpLastResponse (@response$)
	    ? "Login failure "
			? response$
	    QUIT ($$EX_LOGIN)
		END IF
		
		INC logged_in
	END IF

END FUNCTION

FUNCTION GetNextArg (index, argc, @argv$[], @arg$)

	IF index+1 > argc THEN GOTO quit
	IF INSTR(argv$[index+1], "-") THEN GOTO quit
	arg$ = argv$[index+1]
	RETURN
	
quit:	
	? "Syntax error"
	Usage ()
	QUIT ($$EX_SYNTAX)
	
END FUNCTION

FUNCTION ProcessFile (file$)

	SHARED logged_in
	SHARED mode
	SHARED action
	SHARED socket
	SHARED wildcard
	
	STATI64 info
	GIANT fsz

	FtpConnection ()
	FtpSetCallBack (0)
	FtpSetBytesTransferred (0)
	FtpSetFilesize (0)
	
	IF (action == $$FTP_SEND) || (action == $$FTP_GET) || (action == $$FTP_RM) THEN
		IF (action == $$FTP_SEND) THEN
			DIM args[1]
			args[0] = &file$
			args[1] = &info
			ret = XstCall ("_stati64", "msvcrt.dll", @args[])
'	    IF _stati64(&file$, &info) == -1 THEN
			IF ret == -1 THEN
	    	? "Error getting "; file$; " info"
				RETURN $$TRUE
			END IF

	    IF info.st_mode & $$_S_IFMT == $$_S_IFDIR THEN  ' file$ is a directory name
				IF FtpMkdir (file$, socket) THEN
					? "mkdir "; file$; " failed"
					FtpLastResponse (@response$)
					? response$
					RETURN $$TRUE
				ELSE
					? "Directory "; file$; " created"
					RETURN
				END IF
	    END IF
			
      fsz = info.st_size

    ELSE
			IFZ wildcard THEN
				DIM filelist$[0]
				filelist$[0] = file$
			ELSE
				IF FtpAccess (file$, $$FTPLIB_DIR, $$FTPLIB_ASCII, socket, @socketdata) THEN
					? "FtpXfr: FtpAccess error"
					XinSocketGetStatus (socketdata, @remote, @syssocket, @syserror, @status, @socketType, @readbytes, @writebytes)
					IF remote THEN XinSocketClose (remote)
					IF socketdata THEN XinSocketClose (socketdata)
					? "Error requesting directory of "; file$
					FtpLastResponse (@response$)
					? response$
					RETURN $$TRUE
				END IF	
	
				XinSocketGetStatus (socketdata, @remote, @syssocket, @syserror, @status, @socketType, @readbytes, @writebytes)

				DO
					buf$  = NULL$($$FTPLIB_BUFSIZ)
					l = FtpRead (@buf$, $$FTPLIB_BUFSIZ, remote)
					IF l > 0 THEN
						s$ = LEFT$(buf$, l)
						IF RINSTR(s$, ".") THEN       ' check to make sure file has an extension
							filelist$ = filelist$ + s$
						END IF
					END IF
				LOOP WHILE l >= 0
				
				IF remote THEN 
					IF XinSocketClose (remote) THEN
						? "FtpXfer: XinSocketClose error"
					END IF
				END IF
			
				IF socketdata THEN
					IF XinSocketClose (socketdata) THEN
						? "FtpXfer: XinSocketClose error"
					END IF
				END IF
				
				IF ReadResponse ('2', socket, @response$, @response2$) THEN
					? "ProcessFile: ReadResponse error"
					RETURN $$TRUE
				END IF
				
				IF filelist$ THEN XstStringToStringArray (filelist$, @filelist$[])
				
			END IF
		END IF
	END IF

		
  SELECT CASE (action)

    CASE $$FTP_DIR :
			sts = FtpDir ("", file$, socket)

    CASE $$FTP_LIST :
			sts = FtpNlst ("", file$, socket)

    CASE $$FTP_SEND :
			FtpSetCallBack (&LogProgress())
			FtpSetFilesize (fsz)
			sts = FtpPut (file$, file$, mode, socket)
			hStdOut = XioGetStdOut ()
			XioGetConsoleInfo (hStdOut, @bufSizeX, @bufSizeY, @x, @y, @fColors)
			XioSetConsoleCursorPos (hStdOut, 79, y) 
			?
			IFZ sts THEN ? file$; " sent"
				
    CASE $$FTP_GET :
			upp = UBOUND(filelist$[])
			FtpSetCallBack (&LogProgress())

			FOR i = 0 TO upp
				sts = FtpSize (filelist$[i], @fsz, socket)
				IF sts THEN 
					? "Unable to get file size for "; filelist$[i]
					EXIT SELECT
				END IF
				FtpSetFilesize (fsz)
				sts = FtpGet (filelist$[i], filelist$[i], mode, socket)
				
				hStdOut = XioGetStdOut ()
				XioGetConsoleInfo (hStdOut, @bufSizeX, @bufSizeY, @x, @y, @fColors)
				XioSetConsoleCursorPos (hStdOut, 79, y) 
				?
				IFZ sts THEN ? filelist$[i]; " retrieved"
			NEXT i

    CASE $$FTP_RM :
			upp = UBOUND(filelist$[])
			FOR i = 0 TO upp
				sts = FtpDelete (filelist$[i], socket)
				IF sts THEN
					? "Unable to delete "; filelist$[i]
				END IF
				IFZ sts THEN ? filelist$[i]; " deleted"
			NEXT i
			
  END SELECT

  IF sts THEN 
		FtpLastResponse (@response$)
		? "ftp error"
		IF response$ THEN ? response$
	END IF

END FUNCTION

FUNCTION SetUmask (mask$)

	SHARED socket

	FtpConnection ()
	FtpSite (mask$, socket)

END FUNCTION

FUNCTION ChangeDir (root$)

	SHARED socket

  FtpConnection ()
	IF FtpChdir (root$, socket) THEN
		? "Change directory failed"
		FtpLastResponse (@response$)
		? response$
		QUIT ($$EX_REMCMD)
	END IF

END FUNCTION


FUNCTION LogProgress (GIANT filesize, GIANT bytes)

	IFZ filesize THEN RETURN $$TRUE
	
	percent = XLONG((bytes/DOUBLE(filesize)) * 100.0)
	
	hStdOut = XioGetStdOut() 
	XioGetConsoleInfo (hStdOut, @bufSizeX, @bufSizeY, @x, @y, @fColors)
	text$ = STRING$(percent) + "% complete"
	XioPutConsoleText (hStdOut, text$, 0, y)
	ProgressBar (15, y, 60, 0, 100, percent, 176, 219)
	XioSetConsoleCursorPos (hStdOut, x, y) 

END FUNCTION

FUNCTION ProgressBar (row, column, length, min, max, value, emptyChar, filledChar)

	DOUBLE interval

  IF max = 0 && min = 0 THEN RETURN
	IFZ length THEN RETURN
	
	interval = length / DOUBLE(max - min)
	currentSize = (value - min) * interval
	
	hStdOut = XioGetStdOut ()
	XioSetConsoleCursorPos (hStdOut, row, column)

	IF currentSize > 0 THEN
		IF currentSize < max THEN
			empty$ = ""
			dif = length - currentSize
			IF dif > 0 THEN empty$ = CHR$(emptyChar, dif)
			PRINT CHR$(filledChar, currentSize) + empty$
		ELSE
			PRINT CHR$(filledChar, length)
		END IF
	ELSE
		PRINT CHR$(emptyChar, length)
	END IF

END FUNCTION
END PROGRAM